/**
 * AbstractST represents the tree created by the parser.
 * @author Dimitris Michelinakis
 */
public class AbstractST {
    /** Left sub-node in the tree */
    public AbstractST left=null;
    /** Right sub-node in the tree */
    public AbstractST right=null;
    /** Leaf-node data */
    public char data=' ';
    /** Type of node (Leaf/Node) */
    public boolean node;

    /**
     * AbstractST is the first constructor of the parser tree,
     * used for main nodes.
     * @param l AbstractST argument is the left sub-node.
     * @param r AbstractST argument is the right sub-node.
     * @author Dimitris Michelinakis
     */
    public AbstractST(AbstractST l,AbstractST r) {
        /* This is a basic node, so assign our left and right sub-nodes */
        left=l;
        right=r;
        /* Mark this as a basic node */
        node=true;
    }

    /**
     * AbstractST is the second constructor of the parser tree,
     * used for leaf nodes.
     * @param d char argument is the data of the leaf-node.
     * @author Dimitris Michelinakis
     */
    public AbstractST(char d) {
        /* This is a leaf node, so store our data */
        data=d;
        /* Mark this as a leaf node */
        node=false;
    }

    /**
     * printTree is a method which prints the tree in String form.
     * @return a <code>String</code> which represents the parsed tree.
     * @author Dimitris Michelinakis
     */
    public String printTree () {
        return printTree(this);
    }

    /**
     * printTree is the private method which recurses to read all the
     * elements in the tree.
     * @param t AbstractST argument is a AbstractST node or leaf.
     * @return a <code>String</code> which represents the parsed tree.
     * @author Dimitris Michelinakis
     */
    private String printTree (AbstractST t) {
        /* String buffer */
        String str="";
        /* Parenthesis check */
        int par=0;
        /* If this is a leaf, read the character */
        if(t.node==false) {
            str=new Character(t.data).toString();
            /* Grow And/Or characters */
            if(t.data=='/')
                str=str+'\\';
            else if(t.data=='\\')
                str=str+'/';
        }
        /* Go down left sub-node/leaf */
        if(t.left!=null) {
            /* If the next is a node, print a ( */
            if(t.left.node==true) {
                str=str+'(';
                par++;
            }
            str=str+printTree(t.left);
        }
        /* Go down right sub-node/leaf */
        if(t.right!=null) {
            /* If the next is a node, print a ) */
            if(t.left.node==true) {
                str=str+')';
                par--;
            }
            str=str+printTree(t.right);
        }
        /* Add the final parenthesis */
        if(par>0) {
            for(int i=0;i<par;i++)
                str=str+')';
        }
        /* Return final string */
        return str;
    }

}
